/*
 *  gaussian.cpp
 *
 *  Created by Kari Haugsdal
 *
 */

#include "gaussian.h"

void GreyScaleHighestRGB(SDL_Surface *surface, SDL_Surface *copy) {
	
  unsigned char highestRPGValue;

  // aquire pointer to pixel data
  // origin is in top-left corner, and data is organized row/scanline-major  
	pixel * pixels = (pixel*) surface->pixels;
	pixel * copyPixels = (pixel*) copy->pixels;

  int x;
  int y;
  for (x = 0; x < surface->w; x++) {
    for (y = 0; y < surface->h; y++) {
		
		if (pixels[x + y*(surface->w)].g < pixels[x + y*(surface->w)].r) {
			if (pixels[x + y*(surface->w)].b < pixels[x + y*(surface->w)].r) {
				highestRPGValue = pixels[x + y*(surface->w)].r;
			} else {
				highestRPGValue = pixels[x + y*(surface->w)].b;
			}
		}
		else {
			if (pixels[x + y*(surface->w)].b < pixels[x + y*(surface->w)].g) {
				highestRPGValue = pixels[x + y*(surface->w)].g;
			} else {
				highestRPGValue = pixels[x + y*(surface->w)].b;
			}
		}
		
		copyPixels[x + y*(surface->w)].r = highestRPGValue;
		copyPixels[x + y*(surface->w)].g = highestRPGValue;
		copyPixels[x + y*(surface->w)].b = highestRPGValue;
		
    }
  }
  
  int xx;
  int yy;
  for (xx = 0; xx < surface->w; xx++) {
    for (yy = 0; yy < surface->h; yy++) {

		pixels[xx + yy*(surface->w)].b = copyPixels[xx + yy*(surface->w)].b;
		pixels[xx + yy*(surface->w)].r = copyPixels[xx + yy*(surface->w)].r;
		pixels[xx + yy*(surface->w)].g = copyPixels[xx + yy*(surface->w)].g;
    }
  }
}


/* Nois reduction */
void GaussianSmoothing(SDL_Surface *surface, SDL_Surface *copy) {

  int sum = 0;
  
  // aquire pointer to pixel data
  // origin is in top-left corner, and data is organized row/scanline-major
  pixel * pixels = (pixel*) surface->pixels;
  pixel * copyPixels = (pixel*) copy->pixels;

  int x;
  int y;
  for (x = 0; x < surface->w; x++) {
    for (y = 0; y < surface->h; y++) {

		sum = sum + (pixels[(x-1) + (y-1)*(surface->w)].b*1);
		sum = sum + (pixels[(x-1) + y*(surface->w)].b*2);
		sum = sum + (pixels[(x-1) + (y+1)*(surface->w)].b*1);
		sum = sum + (pixels[x + (y-1)*(surface->w)].b*2);
		sum = sum + (pixels[x + y*(surface->w)].b*3);
		sum = sum + (pixels[x + (y+1)*(surface->w)].b*2);
		sum = sum + (pixels[(x+1) + (y-1)*(surface->w)].b*1);
		sum = sum + (pixels[(x+1) + y*(surface->w)].b*2);
		sum = sum + (pixels[(x+1) + (y+1)*(surface->w)].b*1);
		
		sum = sum/15;

		copyPixels[x + y*(surface->w)].r = sum;
		copyPixels[x + y*(surface->w)].g = sum;
		copyPixels[x + y*(surface->w)].b = sum;

		sum = 0;
    }
  }
  
  int xx;
  int yy;
  for (xx = 0; xx < surface->w; xx++) {
    for (yy = 0; yy < surface->h; yy++) {

		pixels[xx + yy*(surface->w)].b = copyPixels[xx + yy*(surface->w)].b;
		pixels[xx + yy*(surface->w)].r = copyPixels[xx + yy*(surface->w)].r;
		pixels[xx + yy*(surface->w)].g = copyPixels[xx + yy*(surface->w)].g;
    }
  }
  
}

/**

int compare(const void * a, const void * b)
{
  return ( *(int*)a - *(int*)b );
}

void GreyScaleMeanRGB(SDL_Surface *surface) {

  unsigned char meanRPGValue;

  // aquire pointer to pixel data
  // origin is in top-left corner, and data is organized row/scanline-major
  pixel * pixels = (pixel*) surface->pixels;

  int x;
  int y;
  for (x = 0; x < surface->w; x++) {
    for (y = 0; y < surface->h; y++) {
	
		if (pixels[x + y*(surface->w)].g < pixels[x + y*(surface->w)].r) {
			if (pixels[x + y*(surface->w)].g < pixels[x + y*(surface->w)].b) {
				if (pixels[x + y*(surface->w)].r < pixels[x + y*(surface->w)].b) {
					meanRPGValue = pixels[x + y*(surface->w)].r;
				} else {
					meanRPGValue = pixels[x + y*(surface->w)].b;
				}
			} else {
				meanRPGValue = pixels[x + y*(surface->w)].g;
			}
		}
		else {
			if (pixels[x + y*(surface->w)].r < pixels[x + y*(surface->w)].b) {
				if (pixels[x + y*(surface->w)].g < pixels[x + y*(surface->w)].b) {
					meanRPGValue = pixels[x + y*(surface->w)].g;
				} else {
					meanRPGValue = pixels[x + y*(surface->w)].b;
				}
			} else {
				meanRPGValue = pixels[x + y*(surface->w)].r;
			}
		}

		pixels[x + y*(surface->w)].a = meanRPGValue;
		
    }
  }
  
  int xx;
  int yy;
  for (xx = 0; xx < surface->w; xx++) {
    for (yy = 0; yy < surface->h; yy++) {

		pixels[xx + yy*(surface->w)].b = pixels[xx + yy*(surface->w)].a;
		pixels[xx + yy*(surface->w)].r = pixels[xx + yy*(surface->w)].a;
		pixels[xx + yy*(surface->w)].g = pixels[xx + yy*(surface->w)].a;
    }
  }
  
}

// Nois reduction
void SimpleSmoothen(SDL_Surface *surface) {

  int sum = 0;
  
  // aquire pointer to pixel data
  // origin is in top-left corner, and data is organized row/scanline-major
  pixel * pixels = (pixel*) surface->pixels;

  int x;
  int y;
  for (x = 1; x < surface->w-1; x++) {
    for (y = 1; y < surface->h-1; y++) {

		sum = sum + pixels[(x-1) + (y-1)*(surface->w)].b;
		sum = sum + pixels[(x-1) + y*(surface->w)].b;
		sum = sum + pixels[(x-1) + (y+1)*(surface->w)].b;
		sum = sum + pixels[x + (y-1)*(surface->w)].b;
		sum = sum + pixels[x + y*(surface->w)].b;
		sum = sum + pixels[x + (y+1)*(surface->w)].b;
		sum = sum + pixels[(x+1) + (y-1)*(surface->w)].b;
		sum = sum + pixels[(x+1) + y*(surface->w)].b;
		sum = sum + pixels[(x+1) + (y+1)*(surface->w)].b;
		
		sum = sum/9;

		pixels[x + y*(surface->w)].a = sum;

		sum = 0;
    }
  }
  
  int xx;
  int yy;
  for (xx = 1; xx < surface->w-1; xx++) {
    for (yy = 1; yy < surface->h-1; yy++) {

		pixels[xx + yy*(surface->w)].b = pixels[xx + yy*(surface->w)].a;
		pixels[xx + yy*(surface->w)].r = pixels[xx + yy*(surface->w)].a;
		pixels[xx + yy*(surface->w)].g = pixels[xx + yy*(surface->w)].a;
    }
  }
  
}


// Do some image-processing to manipulate the image
void SmoothenMedian(SDL_Surface *surface) {

  int Mask[9];
  int Mean;
  
  // aquire pointer to pixel data
  // origin is in top-left corner, and data is organized row/scanline-major
  pixel * pixels = (pixel*) surface->pixels;

  int x;
  int y;
  for (x = 1; x < surface->w-1; x++) {
    for (y = 1; y < surface->h-1; y++) {

		Mask[0] = pixels[(x-1) + (y-1)*(surface->w)].b;
		Mask[1] = pixels[(x-1) + y*(surface->w)].b;
		Mask[2] = pixels[(x-1) + (y+1)*(surface->w)].b;
		Mask[3] = pixels[x + (y-1)*(surface->w)].b;
		Mask[4] = pixels[x + y*(surface->w)].b;
		Mask[5] = pixels[x + (y+1)*(surface->w)].b;
		Mask[6] = pixels[(x+1) + (y-1)*(surface->w)].b;
		Mask[7] = pixels[(x+1) + y*(surface->w)].b;
		Mask[8] = pixels[(x+1) + (y+1)*(surface->w)].b;
		
		qsort (Mask, 9, sizeof(int), compare);
		Mean = Mask[4];

		pixels[x + y*(surface->w)].a = Mean;

    }
  }
  
  int xx;
  int yy;
  for (xx = 1; xx < surface->w-1; xx++) {
    for (yy = 1; yy < surface->h-1; yy++) {

		pixels[xx + yy*(surface->w)].b = pixels[xx + yy*(surface->w)].a;
		pixels[xx + yy*(surface->w)].r = pixels[xx + yy*(surface->w)].a;
		pixels[xx + yy*(surface->w)].g = pixels[xx + yy*(surface->w)].a;
    }
  }
  
}

**/
