/*
 *  gradient.cpp
 *
 *  Created by Kari Haugsdal
 *
 */

#include <iostream>
#include "gradient.h"

// Calculates the gradient magnitude and angel using a mask
void Gradient(SDL_Surface *surface, SDL_Surface *copy) {

	int magnitude = 0;
	float angle = 0;
	float degreeAngle = 0;
	
	// aquire pointer to pixel data
	// origin is in top-left corner, and data is organized row/scanline-major
	pixel * pixels = (pixel*) surface->pixels;
	pixel * copyPixels = (pixel*) copy->pixels;
  
	//The Sobel masks
	//int mask[2][3][3] = { { { -1, 0, 1 }, { -2, 0, 2 }, { -1, 0, 1 } },
	//					{ { 1, 2, 1 }, { 0, 0, 0 }, { -1, -2, -1 } }};
	
	//The Diagonal Sobelt masks
	//int diagonalMask[2][3][3] = { { { 0, 1, 2 }, { -1, 0, 1 }, { -2, -1, 0 } },
	//							  { { -2, -1, 0 }, { -1, 0, 1 }, { 0, 1, 2 } }};
						
	//The Prewitt masks, Works best
	int mask[2][3][3] = { { { -1, 0, 1 }, { -1, 0, 1 }, { -1, 0, 1 } },
						{ { 1, 1, 1 }, { 0, 0, 0 }, { -1, -1, -1 } }};
						
	//The Diagonal Prewitt masks
	int diagonalMask[2][3][3] = { { { 0, 1, 1 }, { -1, 0, 1 }, { -1, -1, 0 } },
								  { { -1, -1, 0 }, { -1, 0, 1 }, { 0, 1, 1 } }};

	//loop through the pixels
	for (int x = 1; x < surface->w-1; x++) {
		for (int y = 1; y < surface->h-1; y++) {
		
			//Looping through the neighbours of the current pixel
			float sumX = 0;
			float sumY = 0;
			float sumD1 = 0;
			float sumD2 = 0;
			for (int i = -1; i < 2; i++) {
				for (int j = -1; j < 2; j++) {
							
					//The picture .r .b and .g value are equal in all pixels after grayscaling, so we just need to loook at one of them
					int pixelVal = pixels[(x + i) + (y + j)* (surface->w)].g;
					int maskValX = mask[0][j + 1][i + 1];
					int maskValY = mask[1][j + 1][i + 1];
					int maskValD1 = diagonalMask[0][j + 1][i + 1];
					int maskValD2 = diagonalMask[1][j + 1][i + 1];
					
					sumX = sumX + (pixelVal * maskValX);
					sumY = sumY + (pixelVal * maskValY);
					sumD1 = sumD1 + (pixelVal * maskValD1);
					sumD2 = sumD2 + (pixelVal * maskValD2);
					
				}
			}

			//magnitude = abs(sumX) + abs(sumY);			//vanlig
			
			//int magnitudeV = abs(sumX) + abs(sumY);
			//int magnitudeD = abs(sumD1) + abs(sumD2);
			//magnitude = (magnitudeV/2) + (magnitudeD/2);	//halvparten av hver
		
			//Eller, velg høyeste! Funker kjempebra!
			if( abs(sumX) > abs(sumY) && abs(sumX) > abs(sumD1) && abs(sumX) > abs(sumD2)){
				//sumX storst
				magnitude =  abs(sumX);
				//magnitude =  abs(sumX) + abs(sumY);
			}
			else if(abs(sumY) > abs(sumD1) && abs(sumX) > abs(sumD2)){
				//sumY storst
				magnitude =  abs(sumY);
				//magnitude =  abs(sumY) + abs(sumX);
			}
			else if(abs(sumD1) > abs(sumD2)){
				//sumD1 storst
				magnitude =  abs(sumD1);
				//magnitude =  abs(sumD1) + abs(sumD2);
			}
			else if(abs(sumD2) > abs(sumD1)){
				//sumD2 storst
				magnitude =  abs(sumD2);
				//magnitude =  abs(sumD2) + abs(sumD1);
			}
			else{ //de er like
				int magnitudeV = abs(sumX) + abs(sumY);
				int magnitudeD = abs(sumD1) + abs(sumD2);
				magnitude = (magnitudeV/2) + (magnitudeD/2);	//halvparten av hver
			}
								
			copyPixels[x + y*(surface->w)].r = magnitude;
			copyPixels[x + y*(surface->w)].g = magnitude;
			copyPixels[x + y*(surface->w)].b = magnitude;
			
			angle = atan2(sumY, sumX);		//radians
			degreeAngle = angle*(180/PI);	//degrees
				
			//Round of and set the .a angle 
			if ( (degreeAngle <= 157.5 && degreeAngle >= 112.5) || (degreeAngle <= -22.5 && degreeAngle >= -67.5)) {
				//-45, 135 degrees
				copyPixels[x + y*(surface->w)].a = 135;
			}
			else if ( (degreeAngle <= 112.5 && degreeAngle >= 67.5) || (degreeAngle <= -67.5 && degreeAngle >= -112.5)) {
				//vertical, 90 degrees
				copyPixels[x + y*(surface->w)].a = 90;
			}
			else if ( (degreeAngle <= 67.5 && degreeAngle >= 22.5) || (degreeAngle <= -112.5 && degreeAngle >= -157.5) ) {
				//45 degree
				copyPixels[x + y*(surface->w)].a = 45;
			}
			else{
				//horizontal, 0 and 180 degree
				copyPixels[x + y*(surface->w)].a = 0;
			}
			
			sumX = 0;
			sumY = 0;
			magnitude = 0;
						
		}
	}//loop through the pixels DONE

	for (int x = 0; x < surface->w; x++) {
		for (int y = 0; y < surface->h; y++) {
			
			pixels[x + y*(surface->w)].a = copyPixels[x + y*(surface->w)].a;
			pixels[x + y*(surface->w)].r = copyPixels[x + y*(surface->w)].r;
			pixels[x + y*(surface->w)].g = copyPixels[x + y*(surface->w)].g;
			pixels[x + y*(surface->w)].b = copyPixels[x + y*(surface->w)].b;
			
		}
	}

}


/** 5 x 5 masks, was worse

// Calculates the gradient magnitude and angel using a mask
void GradientMagnitude(SDL_Surface *surface, SDL_Surface *copy) {

	int magnitude = 0;
	float angle = 0;
	float degreeAngle = 0;
	
	// aquire pointer to pixel data
	// origin is in top-left corner, and data is organized row/scanline-major
	pixel * pixels = (pixel*) surface->pixels;
	pixel * copyPixels = (pixel*) copy->pixels;
  
	//The Sobel masks
	//int mask[2][3][3] = { { { -1, 0, 1 }, { -2, 0, 2 }, { -1, 0, 1 } },
	//					{ { 1, 2, 1 }, { 0, 0, 0 }, { -1, -2, -1 } }};
	
	//The Diagonal Sobelt masks
	//int diagonalMask[2][3][3] = { { { 0, 1, 2 }, { -1, 0, 1 }, { -2, -1, 0 } },
	//							  { { -2, -1, 0 }, { -1, 0, 1 }, { 0, 1, 2 } }};
						
	//The Prewitt masks
	//Rikitg rekkefolge?
	int mask[2][5][5] = { { { -1, 0, 0, 0, 1 }, { -1, 0, 0, 0, 1 }, { -1, 0, 0, 0, 1 }, { -1, 0, 0, 0, 1 }, { -1, 0, 0, 0, 1 }},
						  { { -1, -1, -1, -1, -1 }, { 0, 0, 0, 0, 0 }, { 0, 0, 0, 0, 0 }, { 0, 0, 0, 0, 0 }, { 1, 1, 1, 1, 1 } }};
						  
	//The Diagonal Prewitt masks
	int diagonalMask[2][5][5] = { { { 0, 0, 1, 1, 1 }, { 0, 0, 0, 1, 1 }, { -1, 0, 0, 0, 1 }, { -1, -1, 0, 0, 0 }, { -1, -1, -1, 0, 0 } },
								  { { -1, -1, -1, 0, 0 }, { -1, -1, 0, 0, 0 }, { -1, 0, 0, 0, -1 }, { 0, 0, 0, 1, 1 }, { 0, 0, 1, 1, 1 } }};

	//loop through the pixels
	for (int x = 1; x < surface->w-1; x++) {
		for (int y = 1; y < surface->h-1; y++) {
		
			//Looping through the neighbours of the current pixel
			float sumX = 0;
			float sumY = 0;
			float sumD1 = 0;
			float sumD2 = 0;
			for (int i = -2; i < 3; i++) {
				for (int j = -2; j < 3; j++) {
							
					//The picture .r .b and .g value are equal in all pixels after grayscaling, so we just need to loook at one of them
					int pixelVal = pixels[(x + i) + (y + j)* (surface->w)].r;
					int maskValX = mask[0][j + 2][i + 2];
					int maskValY = mask[1][j + 2][i + 2];
					int maskValD1 = diagonalMask[0][j + 2][i + 2];
					int maskValD2 = diagonalMask[1][j + 2][i + 2];
					
					sumX = sumX + (pixelVal * maskValX);
					sumY = sumY + (pixelVal * maskValY);
					sumD1 = sumD1 + (pixelVal * maskValD1);
					sumD2 = sumD2 + (pixelVal * maskValD2);
					
				}
			}

			int magnitude = abs(sumX) + abs(sumY); //vanlig
			//int magnitudeV = abs(sumX) + abs(sumY);
			//int magnitudeD = abs(sumD1) + abs(sumD2);
			//magnitude = (magnitudeV/2) + (magnitudeD/2); //hmmm ja? Riktig?
			//Eller, velg høyeste!
								
			copyPixels[x + y*(surface->w)].r = magnitude;
			copyPixels[x + y*(surface->w)].g = magnitude;
			copyPixels[x + y*(surface->w)].b = magnitude;
			
			angle = atan2(sumY, sumX);		//radians
			degreeAngle = angle*(180/PI);	//degrees
				
			//Round of and set the .a angle 
			if ( (degreeAngle <= 157.5 && degreeAngle >= 112.5) || (degreeAngle <= -22.5 && degreeAngle >= -67.5)) {
				//-45, 135 degrees
				copyPixels[x + y*(surface->w)].a = 135;
			}
			else if ( (degreeAngle <= 112.5 && degreeAngle >= 67.5) || (degreeAngle <= -67.5 && degreeAngle >= -112.5)) {
				//vertical, 90 degrees
				copyPixels[x + y*(surface->w)].a = 90;
			}
			else if ( (degreeAngle <= 67.5 && degreeAngle >= 22.5) || (degreeAngle <= -112.5 && degreeAngle >= -157.5) ) {
				//45 degree
				copyPixels[x + y*(surface->w)].a = 45;
			}
			else{
				//horizontal, 0 and 180 degree
				copyPixels[x + y*(surface->w)].a = 0;
			}
			
			sumX = 0;
			sumY = 0;
			magnitude = 0;
						
		}
	}//loop through the pixels DONE

	for (int x = 0; x < surface->w; x++) {
		for (int y = 0; y < surface->h; y++) {
			
			pixels[x + y*(surface->w)].a = copyPixels[x + y*(surface->w)].a;
			pixels[x + y*(surface->w)].r = copyPixels[x + y*(surface->w)].r;
			pixels[x + y*(surface->w)].g = copyPixels[x + y*(surface->w)].g;
			pixels[x + y*(surface->w)].b = copyPixels[x + y*(surface->w)].b;
			
		}
	}

}

**/
