/*
 *  hysteresis_thresholding.cpp
 *
 *  Created by Kari Haugsdal
 *
 */

#include "hysteresis_thresholding.h"

/* Hysteresis Thresholding to connect the edges */
void HysteresisThresholding(SDL_Surface *surface, SDL_Surface *surfaceNH, SDL_Surface *surfaceNL) {
  
  // aquire pointer to pixel data
  pixel * pixels = (pixel*) surface->pixels;
  pixel * pixelsNH = (pixel*) surfaceNH->pixels;
  pixel * pixelsNL = (pixel*) surfaceNL->pixels;
  
  //Sett the thresholds
  int highT = FindGoodHighThreshold(surface);
  int lowT = FindGoodLowThreshold(surface);
  
  int x;
  int y;
  for (x = 1; x < surface->w-1; x++) {		//bedre med 1 her av en eller annen grunn
    for (y = 1; y < surface->h-1; y++) {

		if ( pixels[x + y*(surface->w)].b >= highT) {
			pixelsNH[x + y*(surfaceNH->w)].b = pixels[x + y*(surface->w)].b;
			pixelsNH[x + y*(surfaceNH->w)].r = pixels[x + y*(surface->w)].r;
			pixelsNH[x + y*(surfaceNH->w)].g = pixels[x + y*(surface->w)].g;
		}
		else {
			pixelsNH[x + y*(surfaceNH->w)].b = 0;
			pixelsNH[x + y*(surfaceNH->w)].r = 0;
			pixelsNH[x + y*(surfaceNH->w)].g = 0;
		}
		
		if ( pixels[x + y*(surface->w)].b >= lowT) {
			pixelsNL[x + y*(surfaceNL->w)].b = pixels[x + y*(surface->w)].b;
			pixelsNL[x + y*(surfaceNL->w)].r = pixels[x + y*(surface->w)].r;
			pixelsNL[x + y*(surfaceNL->w)].g = pixels[x + y*(surface->w)].g;
		}
		else {
			pixelsNL[x + y*(surfaceNL->w)].b = 0;
			pixelsNL[x + y*(surfaceNL->w)].r = 0;
			pixelsNL[x + y*(surfaceNL->w)].g = 0;
		}

	}
  }
  
  
  //Gjore ferdig NL
  int xx;
  int yy;
  for (xx = 0; xx < surface->w; xx++) {
    for (yy = 0; yy < surface->h; yy++) {

		if ( pixelsNH[xx + yy*(surfaceNH->w)].b > 0) {
			pixelsNL[xx + yy*(surfaceNL->w)].b = 0;
			pixelsNL[xx + yy*(surfaceNL->w)].r = 0;
			pixelsNL[xx + yy*(surfaceNL->w)].g = 0;
		}

    }
  }
  
  //connectivity analysis, using 8-connectivity
  int xc;
  int yc;
  for (xc = 1; xc < surface->w-1; xc++) {
    for (yc = 1; yc < surface->h-1; yc++) {

		if ( pixelsNH[xc + yc*(surfaceNH->w)].b > 0) { // && unvisited?
		
			if (pixelsNL[(xc-1) + (yc-1)*(surfaceNL->w)].b > 0) {
				pixelsNL[(xc-1) + (yc-1)*(surfaceNL->w)].a = 1;	//marked as visited!
			}
			if (pixelsNL[(xc-1) + yc*(surfaceNL->w)].b > 0) {
				pixelsNL[(xc-1) + yc*(surfaceNL->w)].a = 1;
			}
			if (pixelsNL[(xc-1) + (yc+1)*(surfaceNL->w)].b > 0) {
				pixelsNL[(xc-1) + (yc+1)*(surfaceNL->w)].a = 1;
			}
			if (pixelsNL[xc + (yc-1)*(surfaceNL->w)].b > 0) {
				pixelsNL[xc + (yc-1)*(surfaceNL->w)].a = 1;
			}
			if (pixelsNL[xc + (yc+1)*(surfaceNL->w)].b > 0) {
				pixelsNL[xc + (yc+1)*(surfaceNL->w)].a = 1;
			}
			if (pixelsNL[(xc+1) + (yc-1)*(surfaceNL->w)].b > 0) {
				pixelsNL[(xc+1) + (yc-1)*(surfaceNL->w)].a = 1;
			}
			if (pixelsNL[(xc+1) + yc*(surfaceNL->w)].b > 0) {
				pixelsNL[(xc+1) + yc*(surfaceNL->w)].a = 1;
			}
			if (pixelsNL[(xc+1) + (yc+1)*(surfaceNL->w)].b > 0) {
				pixelsNL[(xc+1) + (yc+1)*(surfaceNL->w)].a = 1;
			}
			
		}

    }
  }
  
  //append all visited pixels from Gnl to Gnh, and use Gnh further
  int xxx;
  int yyy;
  for (xxx = 0; xxx < surface->w; xxx++) {
    for (yyy = 0; yyy < surface->h; yyy++) {

		if ( pixelsNL[xxx + yyy*(surfaceNL->w)].a == 1) {
			pixelsNH[xxx + yyy*(surfaceNH->w)].b = pixelsNL[xxx + yyy*(surfaceNL->w)].b;
			pixelsNH[xxx + yyy*(surfaceNH->w)].r = pixelsNL[xxx + yyy*(surfaceNL->w)].r;
			pixelsNH[xxx + yyy*(surfaceNH->w)].g = pixelsNL[xxx + yyy*(surfaceNL->w)].g;
		}

    }
  }
  
  //sett inn bilde i surface som brukes videre
  int xs;
  int ys;
  for (xs = 0; xs < surface->w; xs++) {
    for (ys = 0; ys < surface->h; ys++) {

		pixels[xs + ys*(surface->w)].b = pixelsNH[xs + ys*(surfaceNH->w)].b;
		pixels[xs + ys*(surface->w)].r = pixelsNH[xs + ys*(surfaceNH->w)].r;
		pixels[xs + ys*(surface->w)].g = pixelsNH[xs + ys*(surfaceNH->w)].g;
		
    }
  }
  
}

int FindGoodHighThreshold(SDL_Surface *surface){
  int x;
  int y;
  int highV = 0;
  
  pixel * pixels = (pixel*) surface->pixels;
  
  for (x = 1; x < surface->w-1; x++) {
    for (y = 1; y < surface->h-1; y++) {
	
		if ( pixels[x + y*(surface->w)].b >= highV) {
		highV = pixels[x + y*(surface->w)].b;
		}
		if ( pixels[x + y*(surface->w)].g >= highV) {
		highV = pixels[x + y*(surface->w)].g;
		}
		if ( pixels[x + y*(surface->w)].r >= highV) {
		highV = pixels[x + y*(surface->w)].r;
		}
		
	}
  }
  
  double temp = highV / 100;
  //double temp2 = temp * 70;			//The percent of the higher threshold
  double temp2 = temp * 40;	
  
  return temp2;
}

int FindGoodLowThreshold(SDL_Surface *surface){
  int x;
  int y;
  int highV = 0;
  
  pixel * pixels = (pixel*) surface->pixels;
  
  for (x = 1; x < surface->w-1; x++) {
    for (y = 1; y < surface->h-1; y++) {
	
		if ( pixels[x + y*(surface->w)].b >= highV) {
		highV = pixels[x + y*(surface->w)].b;
		}
		if ( pixels[x + y*(surface->w)].g >= highV) {
		highV = pixels[x + y*(surface->w)].g;
		}
		if ( pixels[x + y*(surface->w)].r >= highV) {
		highV = pixels[x + y*(surface->w)].r;
		}

	}
  }
  
  double temp = highV / 100;
  //double temp2 = temp * 30;			//The percent of the lower threshold
  double temp2 = temp * 15;
  
  return temp2;
}