/*
 *  main.cpp
 *
 *  Created by Kari Haugsdal
 */

#include <iostream>
#include <string.h>
#include "gaussian.h"
#include "gradient.h"
#include "nonmaxima_suppression.h"
#include "hysteresis_thresholding.h"
#include "thinning.h"
#include "m_connectivity.h"
#include "local_edge_linking.h"
#include "tools.h"
#include "linking.h"

#include "SDL.h"

void ModifyImage(SDL_Surface *surface, SDL_Surface *copy, SDL_Surface *copy2){
	
	GreyScaleHighestRGB(surface, copy);
	GaussianSmoothing(surface, copy);
	
	Gradient(surface, copy);
	
	NonmaximaSuppression(surface, copy);

	HysteresisThresholding(surface, copy, copy2);
	
	//pre-project (some bug fiksing done)
	//-------------------------------------------
	//master

	Thinning(surface, copy);
	MConnectivity(surface, copy);
	MakeFrame(surface);
	
	Linking(surface, copy);
	
	GetRidOfSmallEdges(surface, copy, 20);
	
	LinkGaps(surface, copy, 10);
	
	//ColorEdges(surface, copy); //tool for viewing results easily
}

int main(int argc, char *argv[]) {

	//const char* pictureName = "NTNU.bmp";

	//taking the picture path as input
	std::cout << "Please enter picture path: ";
	std::string name;
	std::cin >> name;
	const char* pictureName = name.c_str();

	//input event (like mouse and keyboard)
	SDL_Event event;

	const int SCREEN_DEPTH  =  32;

	//load bitmap code
    SDL_Surface *screen = NULL; // The surface on which we will load the screen
    SDL_Surface *bitmap = NULL; // The surface on which we will load the bitmap
    SDL_Rect offset; // The rectangle (target area) to which we will load the bitmap

    // Initialize SDL
    SDL_Init(SDL_INIT_VIDEO);

    // Load an image
    bitmap = SDL_LoadBMP (pictureName);

	//getting height and width
	int Screen_w = bitmap->w;
	int Screen_h = bitmap->h;

    // Initialise the screen, by passing the screen width, height, depth, and surface type
	screen = SDL_SetVideoMode(Screen_w, Screen_h, SCREEN_DEPTH, SDL_SWSURFACE);

    // Set up the offset variables, where we will draw the bitmap to
    offset.x = 0;
    offset.y = 0;
    offset.h = Screen_h;
    offset.w = Screen_w;

    // Apply the bitmap to the screen
    SDL_BlitSurface (bitmap, NULL, screen, &offset);

	// Surface Two
    SDL_Surface *screen2 = SDL_ConvertSurface(screen, screen->format, SDL_SWSURFACE);

	// Surface Three
	SDL_Surface *screen3 = SDL_ConvertSurface(screen, screen->format, SDL_SWSURFACE);

	//Mainpart! Modify the image
	ModifyImage(screen, screen2, screen3);

    // update the screen
    SDL_Flip(screen);

	// save modified image back to file
    SDL_SaveBMP(screen, "output.bmp");

	//code to exit window
	 // wait for mouse and keyboard events
    while ( SDL_WaitEvent(&event) >= 0 ) {
		switch (event.type) {
		  // "quit"-button pressed
			case SDL_QUIT: {
			  // cleanup and exit
			  SDL_Quit();
			  return 0;
			 }
			// any key pressed
			case SDL_KEYDOWN: {
			  // cleanup and exit
			  SDL_Quit();
			  return 0;
			}
		 }
	}

	return 0;
}
